#ifndef CONTENTMANAGER_H
#define CONTENTMANAGER_H

#include <vector>
#include <d3d11.h>
#include <d3dx11.h>
#include "D3D11_Core.h"
#include "ShaderModel.h"
#include "Effects/VSPSEffect.h"
#include "Texture.h"

/// <summary>
/// Takes care loading and unloading the resources
/// </summary>
/// <remarks>
/// For now this is a naive implementation that performs the loading operation
/// each time it is requested
/// </remarks>
class ContentManager
{
  public:
    /// <summary>
    /// Creates a content manager
    /// </summary>
    /// <param name="d3d11">
    /// D3D11 data needed to load content
    /// </param>
    ContentManager(D3D11_Core& d3d11);
    
    /// <summary>
    /// Cleans up the content manager
    /// </summary>
    ~ContentManager();
    
    /// <summary>
    /// Creates an effect that uses the vertex and pixel shader stages of the
    /// rendering pipeline
    /// </summary>
    /// <param name="file">
    /// path to the file that contains the shader code for both the vertex and
    /// pixel shaders
    /// </param>
    /// <param name="model">
    /// which shader model to use
    /// </param>
    /// <param name="layout">
    /// layout of inputs to the vertex shader
    /// </param>
    /// <param name="num_layout_entries">
    /// number of entries in layout
    /// </param>
    /// <param name="buffer_widths">
    /// number of bytes in each constant buffer to create
    /// </param>
    /// <returns>
    /// pointer to the loaded effect
    /// NULL if an error occurred
    /// </returns>
    VSPSEffect* LoadEffect(WCHAR* file,ShaderModel model,
      D3D11_INPUT_ELEMENT_DESC layout[],UINT num_layout_entries,
      const std::vector<UINT>& buffer_widths);
    
    /// <summary>
    /// Loads a texture
    /// </summary>
    /// <param name="file">
    /// path to the file that contains the texture
    /// </param>
    /// <returns>
    /// pointer to the loaded texture
    /// NULL if an error occurred
    /// </returns>
    Texture* LoadTexture(WCHAR* file);
    
    /// <summary>
    /// Loads a texture
    /// </summary>
    /// <param name="file">
    /// path to the file that contains the texture
    /// </param>
    /// <returns>
    /// pointer to the loaded texture
    /// NULL if an error occurred
    /// </returns>
    Texture* LoadTexture(const char* file);
    
    /// <summary>
    /// Informs the content manager that the effect can be unloaded
    /// </summary>
    /// <param name="effect">
    /// effect that can be unloaded
    /// </param>
    void UnloadEffect(VSPSEffect* effect);
    
    /// <summary>
    /// Informs the content manager that the texture can be unloaded
    /// </summary>
    /// <param name="tex">
    /// texture that can be unloaded
    /// </param>
    void UnloadTexture(Texture* tex);
    
  private:
    // disabled
    ContentManager(const ContentManager& cpy);
    ContentManager& operator=(const ContentManager& cpy);
    
    /// <summary>
    /// Compiles a shader from a file
    /// </summary>
    /// <param name="file">
    /// path to the file to load
    /// </param>
    /// <param name="entry_point">
    /// function name in the shader that is the entry point to the shader to
    /// compile
    /// </param>
    /// <param name="model">
    /// d3d11 string description of the shader model to use for compilation
    /// </param>
    /// <param name="blob">
    /// where to put the compiled shader
    /// </param>
    /// <returns>
    /// true if the shader was successfully compiled
    // false otherwise
    /// </returns>
    bool CompileShader(WCHAR* file,const char* entry_point,const char* model,
      ID3DBlob** blob);
    
    /// <summary>
    /// D3D11 data needed to load content
    /// </summary>
    D3D11_Core& m_d3d11;
};

#endif /* CONTENTMANAGER_H */
