#ifndef TEXTUREDEFFECT_H
#define TEXTUREDEFFECT_H

#include <vector>
#include "Effect.h"
#include "D3D11_Core.h"
#include "Texture.h"
#include "TextureSampler.h"

class ContentManager;

/// <summary>
/// For effects that use vertex and pixel shaders
/// </summary>
class VSPSEffect : public Effect
{
  public:
    /// <summary>
    /// Creates an effect
    /// </summary>
    /// <param name="vertex">
    /// vertex shader
    /// </param>
    /// <param name="layout">
    /// layout for vertex shader input
    /// </param>
    /// <param name="pixel">
    /// pixel shader
    /// </param>
    /// <param name="buffers">
    /// constant buffers for the effect
    /// </param>
    VSPSEffect(ID3D11VertexShader* vertex,ID3D11InputLayout* layout,
      ID3D11PixelShader* pixel,const std::vector<ID3D11Buffer*>& buffers);
    
    /// <summary>
    /// Cleans up the effect
    /// </summary>
    virtual ~VSPSEffect();
    
    /// <summary>
    /// Setups the effect to be used for rendering
    /// <summary>
    /// <param name="context">
    /// D3D11 context to apply the effect to
    /// </param>
    void MakeActive(ID3D11DeviceContext* context);
    
    /// <summary>
    /// Sets the value of the desired constant buffer
    /// <summary>
    /// <param name="context">
    /// d3d11 context to update the value of the buffer in
    /// </param>
    /// <param name="id">
    /// 0 based index of which constant buffer to set the value of
    /// </param>
    /// <param name="val">
    /// pointer to the data to update the buffer with.  The data must be the
    /// same size as was used to create the buffer
    /// <remarks>
    /// any matrices that are pointed to by this need to be transposed before
    /// calling this
    /// </remarks>
    /// </param>
    void SetConstantBuffer(ID3D11DeviceContext* context,UINT id,
      const void* val);
    
    /// <summary>
    /// Supplies the specified texture to the pixel shader
    /// </summary>
    /// <param name="context">
    /// d3d11 context to supply the texture to
    /// </param>
    /// <param name="tex">
    /// texture to supply to the pixel shader
    /// </param>
    /// <param name="index">
    /// which texture index in the shader the texture should use
    /// </param>
    void UseTexture(ID3D11DeviceContext* context,Texture* tex,UINT index);
    
    /// <summary>
    /// Supplies the specified texture sampler to the pixel shader
    /// </summary>
    /// <param name="context">
    /// d3d11 context to supply the texture sampler to
    /// </param>
    /// <param name="sampler">
    /// texture sampler to supply to the pixel shader
    /// </param>
    /// <param name="index">
    /// which texture index in the shader the texture sampler should be
    /// supplied to
    /// </param>
    void UseTextureSampler(ID3D11DeviceContext* context,
      TextureSampler* sampler,UINT index);
    
  private:
    // disabled
    VSPSEffect(const VSPSEffect& cpy);
    VSPSEffect& operator=(const VSPSEffect& cpy);
    
    // not implemented: until the ContentManager tracks the D3D11 resources
    // directly
    friend ContentManager;
    
    /// <summary>
    /// vertex shader
    /// </summary>
    ID3D11VertexShader* m_vertex;
    
    /// <summary>
    /// layout for vertex shader input
    /// </summary>
    ID3D11InputLayout*  m_vertex_layout;
    
    /// <summary>
    /// pixel shader
    /// </summary>
    ID3D11PixelShader*  m_pixel;
    
    /// <summary>
    /// constant buffers for the effect
    /// </summary>
    /// <remarks>
    /// c style array for easy usage with D3D11's api
    /// </remarks>
    ID3D11Buffer** m_buffers;
    
    /// <summary>
    /// number of buffers in m_buffers
    /// </summary>
    UINT m_num_buffers;
};

#endif /* TEXTUREDEFFECT_H */
