#ifndef GAME_H
#define GAME_H

#include "Window.h"
#include "D3D11_Core.h"
#include "ContentManager.h"

/// <summary>
/// Base class for Game objects that provide the basic framework for a game
/// (i.e. takes care of managing the window, handles time delays for the update
/// cycle, etc)
/// </summary>
class Game
{
  public:
    /// <summary>
    /// creates the game instance, including the window that will be used for
    /// it
    /// </summary>
    Game(WCHAR* title,HINSTANCE instance);
    
    /// <summary>
    /// performs any needed cleanup
    /// </summary>
    virtual ~Game();
    
    /// <summary>
    /// runs the game until it is exited for any reason
    /// </summary>
    void Run();
    
  protected:
    /// <summary>
    /// Updates how often the update/draw cycle should occur
    /// </summary>
    /// <param name="ms">
    /// number of milliseconds between update/draw cycles
    /// </param>
    void SetTimePerFrame(UINT ms);
    
    /// <summary>
    /// Where subclasses should load their content (graphical or otherwise)
    /// </summary>
    /// <param name="content">
    /// content manager to use for loading resources
    /// </param>
    virtual void LoadContent() = 0;
    
    /// <summary>
    /// Where subclasses should unload their content (graphical or otherwise)
    /// just before deallocation of the instance
    /// </summary>
    /// <param name="content">
    /// content manager to use for unloading resources
    /// </param>
    virtual void UnloadContent() = 0;
    
    /// <summary>
    /// Where subclasses should update their game state
    /// </summary>
    /// <param name="ms">
    /// number of milliseconds since the update
    /// </param>
    virtual void Update(UINT ms) = 0;
    
    /// <summary>
    /// Where subclasses should draw the current frame
    /// </summary>
    /// <param name="ms">
    /// number of milliseconds since the last frame
    /// </param>
    virtual void Draw(UINT ms) = 0;
    
    /// <summary>
    /// D3D11 core for m_window
    /// </summary>
    /// <remarks>
    /// subclasses should only access this variable and not set it
    /// </remarks>
    D3D11_Core* m_graphics;
    
    /// <summary>
    /// D3D11 core for m_window
    /// </summary>
    /// <remarks>
    /// should only be accessed between and during LoadContent/UnloadContent
    /// </remarks>
    ContentManager* m_content;
    
    /// <summary>
    /// number of milliseconds to wait between frames
    /// </summary>
    /// <remarks>
    /// constructor defaults this to 60 fps
    /// </remarks>
    UINT m_mspf;
    
  private:
    // disabled
    Game(const Game& cpy);
    Game& operator=(const Game& cpy);
    
    /// <summary>
    /// Performs operations to draw the current frame
    /// </summary>
    /// <param name="ms">
    /// number of milliseconds since the last frame
    /// </param>
    void Render(UINT ms);
    
    /// <summary>
    /// window for the game
    /// </summary>
    Window* m_window;
};

#endif /* GAME_H */
