#include <vector>
#include <sstream>
#include "ContentManager.h"
#include <d3dcompiler.h>
#include "log.h"
using namespace std;

ContentManager::ContentManager(D3D11_Core& d3d11)
:m_d3d11(d3d11)
{
}

ContentManager::~ContentManager()
{
}

VSPSEffect* ContentManager::LoadEffect(WCHAR* file,ShaderModel model,
  D3D11_INPUT_ELEMENT_DESC layout[],UINT num_layout_entries,
  const vector<UINT>& buffer_widths)
{
  ID3D11Device* device = m_d3d11.GetDevice();
  
  // compile the vertex shader
  ID3DBlob* blob = NULL;
  if (!CompileShader(file,"VS",ShaderModelToVertexModel(model),&blob))
  {
    if (log_would_output())
    {
      char* tmp = wchar_to_str(file);
      ostringstream buffer;
      buffer << "Failed to compile vertex shader for \"" << tmp
             << "\", shader model " << ShaderModelToVertexModel(model)
             << '\n';
      log_print(buffer.str().c_str());
      delete[]tmp;
    }
    return NULL;
  }
  
  // create the vertex shader
  ID3D11VertexShader* vertex_shader = NULL;
  if (FAILED(device->CreateVertexShader(blob->GetBufferPointer(),
    blob->GetBufferSize(),NULL,&vertex_shader)))
  {
    if (log_would_output())
    {
      char* tmp = wchar_to_str(file);
      ostringstream buffer;
      buffer << "Failed to create the vertex shader for \"" << tmp
             << "\", shader model " << ShaderModelToVertexModel(model)
             << '\n';
      log_print(buffer.str().c_str());
      delete[]tmp;
    }
    blob->Release();
    return NULL;
  }
  
  // create the input layout
  ID3D11InputLayout* vertex_layout = NULL;
  if (FAILED(device->CreateInputLayout(layout,num_layout_entries,
    blob->GetBufferPointer(),blob->GetBufferSize(),&vertex_layout)))
  {
    if (log_would_output())
    {
      char* tmp = wchar_to_str(file);
      ostringstream buffer;
      buffer << "Failed to create the input layout for \"" << tmp
             << "\", shader model " << ShaderModelToVertexModel(model)
             << '\n';
      log_print(buffer.str().c_str());
      delete[]tmp;
    }
    blob->Release();
    return NULL;
  }
  blob->Release();
  
  // compile the pixel shader
  if (!CompileShader(file,"PS",ShaderModelToPixelModel(model),&blob))
  {
    if (log_would_output())
    {
      char* tmp = wchar_to_str(file);
      ostringstream buffer;
      buffer << "Failed to compile the pixel shader for \"" << tmp
             << "\", shader model " << ShaderModelToPixelModel(model)
             << '\n';
      log_print(buffer.str().c_str());
      delete[]tmp;
    }
    return NULL;
  }
  
  // create the pixel shader
  ID3D11PixelShader* pixel_shader = NULL;
  if (FAILED(device->CreatePixelShader(blob->GetBufferPointer(),
    blob->GetBufferSize(),NULL,&pixel_shader)))
  {
    if (log_would_output())
    {
      char* tmp = wchar_to_str(file);
      ostringstream buffer;
      buffer << "Failed to create the pixel shader for \"" << tmp
             << "\", shader model " << ShaderModelToVertexModel(model)
             << '\n';
      log_print(buffer.str().c_str());
      delete[]tmp;
    }
    blob->Release();
    return NULL;
  }
  blob->Release();
  
  // setup common values for constant buffers
  D3D11_BUFFER_DESC bd;
  ZeroMemory(&bd,sizeof(bd));
  bd.Usage          = D3D11_USAGE_DEFAULT;
  bd.BindFlags      = D3D11_BIND_CONSTANT_BUFFER;
  bd.CPUAccessFlags = 0;
  
  // create the constant buffers
  vector<ID3D11Buffer*> buffers;
  buffers.reserve(buffer_widths.size());
  vector<UINT>::const_iterator widths_it = buffer_widths.begin();
  while (widths_it != buffer_widths.end())
  {
    ID3D11Buffer* buffer = NULL;
    bd.ByteWidth = *widths_it;
    if (SUCCEEDED(device->CreateBuffer(&bd,NULL,&buffer)))
    {
      buffers.push_back(buffer);
    }
    else
    {
      if (log_would_output())
      {
        char* tmp = wchar_to_str(file);
        ostringstream buffer;
        buffer << "Failed to create the constant buffer " << buffers.size()
               << "for \"" << tmp << "\", shader model "
               << ShaderModelToVertexModel(model) << '\n';
        log_print(buffer.str().c_str());
        delete[]tmp;
      }
      
      // clean up the ones that went through already
      vector<ID3D11Buffer*>::iterator buff_it = buffers.begin();
      while (buff_it != buffers.end())
      {
        (*buff_it)->Release();
        
        ++buff_it;
      }
      return NULL;
    }
    
    ++widths_it;
  }

  return new VSPSEffect(vertex_shader,vertex_layout,pixel_shader,buffers);
}

Texture* ContentManager::LoadTexture(WCHAR* file)
{
  ID3D11ShaderResourceView* tex;
  if (FAILED(D3DX11CreateShaderResourceViewFromFile(m_d3d11.GetDevice(),file,
    NULL,NULL,&tex,NULL)))
  {
    if (log_would_output())
    {
      ostringstream buffer;
      buffer << "Failed to load texture \"" << file << "\"\n";
      log_print(buffer.str().c_str());
    }
    return NULL;
  }
  return new Texture(tex);
}

Texture* ContentManager::LoadTexture(const char* file)
{
  ID3D11ShaderResourceView* tex;
  if (FAILED(D3DX11CreateShaderResourceViewFromFileA(m_d3d11.GetDevice(),file,
    NULL,NULL,&tex,NULL)))
  {
    if (log_would_output())
    {
      ostringstream buffer;
      buffer << "Failed to load texture \"" << file << "\"\n";
      log_print(buffer.str().c_str());
    }
    return NULL;
  }
  return new Texture(tex);
}

void ContentManager::UnloadEffect(VSPSEffect* effect)
{
  if (effect != NULL)
  {
    for (UINT i = 0; i < effect->m_num_buffers; ++i)
    {
      effect->m_buffers[i]->Release();
    }
  
    effect->m_vertex_layout->Release();
    effect->m_vertex->Release();
    effect->m_pixel->Release();
  }
}

void ContentManager::UnloadTexture(Texture* tex)
{
  if (tex != NULL)
  {
    tex->m_texture->Release();
  }
}

bool ContentManager::CompileShader(WCHAR* file,LPCSTR entry_point,
  LPCSTR shader_model,ID3DBlob** blob)
{
  bool back = true;

  DWORD shader_flags = D3DCOMPILE_ENABLE_STRICTNESS;
#ifdef _DEBUG 
  shader_flags |= D3DCOMPILE_DEBUG;
#endif /* _DEBUG */

  ID3DBlob* error_blob = NULL;
  back = SUCCEEDED(D3DX11CompileFromFile(file,NULL,NULL,entry_point,
    shader_model, shader_flags,0,NULL,blob,&error_blob,NULL));
  if (!back)
  {
    if (error_blob)
    {
      log_print((char*)error_blob->GetBufferPointer());
      error_blob->Release();
    }
    return back;
  }
  if (error_blob)
  {
    error_blob->Release();
  }

  return back;
}
