#include "D3D11_Core.h"
#include "log.h"

D3D11_Core* D3D11_Core::Create(HWND& wnd)
{
  HRESULT rc = S_OK;
  
  // vars needed for setting up D3D11
  ID3D11Device*           device             = NULL;
  ID3D11DeviceContext*    immediate_context  = NULL;
  IDXGISwapChain*         swap_chain         = NULL;
  ID3D11RenderTargetView* render_target_view = NULL;
  ID3D11Texture2D*        depth_stencil      = NULL;
  ID3D11DepthStencilView* depth_stencil_view = NULL;

  // determine the window width and height
  RECT rect;
  GetClientRect(wnd,&rect);
  UINT width  = rect.right  - rect.left;
  UINT height = rect.bottom - rect.top;

  UINT create_device_flags = 0;
#ifdef _DEBUG
  create_device_flags |= D3D11_CREATE_DEVICE_DEBUG;
#endif

  D3D_DRIVER_TYPE driver_types[] =
  {
    D3D_DRIVER_TYPE_HARDWARE,
    D3D_DRIVER_TYPE_WARP,
    D3D_DRIVER_TYPE_REFERENCE,
  };
  UINT num_driver_types = ARRAYSIZE(driver_types);

  D3D_FEATURE_LEVEL feature_levels[] =
  {
    D3D_FEATURE_LEVEL_11_0,
  };
  UINT num_feature_levels = ARRAYSIZE(feature_levels);

  // create the device and swap chain
  DXGI_SWAP_CHAIN_DESC sd;
  ZeroMemory(&sd,sizeof(sd));
  sd.BufferCount        = 1;
  sd.BufferDesc.Width   = width;
  sd.BufferDesc.Height  = height;
  sd.BufferDesc.Format  = DXGI_FORMAT_R8G8B8A8_UNORM;
  sd.BufferUsage        = DXGI_USAGE_RENDER_TARGET_OUTPUT;
  sd.OutputWindow       = wnd;
  sd.SampleDesc.Count   = 1;
  sd.SampleDesc.Quality = 0;
  sd.Windowed           = TRUE;
  sd.BufferDesc.RefreshRate.Numerator   = 60;
  sd.BufferDesc.RefreshRate.Denominator = 1;
  for (UINT i = 0; i < num_driver_types; ++i)
  {
      D3D_DRIVER_TYPE type = driver_types[i];
      rc = D3D11CreateDeviceAndSwapChain(NULL,type,NULL,
        create_device_flags,feature_levels,num_feature_levels,
        D3D11_SDK_VERSION,&sd,&swap_chain,&device,NULL,&immediate_context);
      if (SUCCEEDED(rc))
      {
        break;
      }
  }
  if (FAILED(rc))
  {
    log_print("Failed to create device and swap chain\n");
    return NULL;
  }

  // Create a render target view
  ID3D11Texture2D* back_buffer = NULL;
  rc = swap_chain->GetBuffer(0,__uuidof(ID3D11Texture2D),
    (LPVOID*)&back_buffer);
  if (FAILED(rc))
  {
    log_print("Failed to retrieve buffer for render target\n");
    return NULL;
  }
  rc = device->CreateRenderTargetView(back_buffer,NULL,&render_target_view);
  back_buffer->Release();
  if (FAILED(rc))
  {
    log_print("Failed to create render target view\n");
    return NULL;
  }

  // Create depth stencil texture
  D3D11_TEXTURE2D_DESC desc_depth;
  ZeroMemory(&desc_depth,sizeof(desc_depth));
  desc_depth.Width              = width;
  desc_depth.Height             = height;
  desc_depth.MipLevels          = 1;
  desc_depth.ArraySize          = 1;
  desc_depth.Format             = DXGI_FORMAT_D24_UNORM_S8_UINT;
  desc_depth.Usage              = D3D11_USAGE_DEFAULT;
  desc_depth.BindFlags          = D3D11_BIND_DEPTH_STENCIL;
  desc_depth.CPUAccessFlags     = 0;
  desc_depth.MiscFlags          = 0;
  desc_depth.SampleDesc.Count   = 1;
  desc_depth.SampleDesc.Quality = 0;
  rc = device->CreateTexture2D(&desc_depth,NULL,&depth_stencil);
  if (FAILED(rc))
  {
    log_print("Failed to create depth stencil texture\n");
    return NULL;
  }

  // Create the depth stencil view
  D3D11_DEPTH_STENCIL_VIEW_DESC desc_dsv;
  ZeroMemory(&desc_dsv,sizeof(desc_dsv));
  desc_dsv.Format             = desc_depth.Format;
  desc_dsv.ViewDimension      = D3D11_DSV_DIMENSION_TEXTURE2D;
  desc_dsv.Texture2D.MipSlice = 0;
  rc = device->CreateDepthStencilView(depth_stencil,&desc_dsv,
    &depth_stencil_view);
  if (FAILED(rc))
  {
    log_print("Failed to create depth stencil view\n");
    return NULL;
  }

  immediate_context->OMSetRenderTargets(1,&render_target_view,
    depth_stencil_view);

  // Setup the viewport
  D3D11_VIEWPORT vp;
  vp.Width    = (FLOAT)width;
  vp.Height   = (FLOAT)height;
  vp.MinDepth = 0.0f;
  vp.MaxDepth = 1.0f;
  vp.TopLeftX = 0;
  vp.TopLeftY = 0;
  immediate_context->RSSetViewports(1,&vp);
  
  return new D3D11_Core(device,immediate_context,swap_chain,render_target_view,
    depth_stencil,depth_stencil_view);
}

D3D11_Core::D3D11_Core(ID3D11Device* device,
  ID3D11DeviceContext* immediate_context,IDXGISwapChain* swap_chain,
  ID3D11RenderTargetView* render_target_view,ID3D11Texture2D* depth_stencil,
  ID3D11DepthStencilView* depth_stencil_view)
:m_device(device),
 m_immediate_context(immediate_context),
 m_swap_chain(swap_chain),
 m_render_target_view(render_target_view),
 m_depth_stencil(depth_stencil),
 m_depth_stencil_view(depth_stencil_view)
{
}

D3D11_Core::~D3D11_Core()
{
  if (m_immediate_context)
  {
    m_immediate_context->ClearState();
  }
  
  if (m_depth_stencil)
  {
    m_depth_stencil->Release();
  }
  if (m_depth_stencil_view)
  {
    m_depth_stencil_view->Release();
  }
  if (m_render_target_view)
  {
    m_render_target_view->Release();
  }
  if (m_swap_chain)
  {
    m_swap_chain->Release();
  }
  if (m_immediate_context)
  {
    m_immediate_context->Release();
  }
  if (m_device)
  {
    m_device->Release();
  }
}

ID3D11Device* D3D11_Core::GetDevice()
{
  return m_device;
}

ID3D11DeviceContext* D3D11_Core::GetImmediateContext()
{
  return m_immediate_context;
}

D3D11_VIEWPORT D3D11_Core::GetViewport(UINT index)
{
  UINT num = D3D11_VIEWPORT_AND_SCISSORRECT_OBJECT_COUNT_PER_PIPELINE;
  D3D11_VIEWPORT viewports[
    D3D11_VIEWPORT_AND_SCISSORRECT_OBJECT_COUNT_PER_PIPELINE];
  m_immediate_context->RSGetViewports(&num,viewports);
  
  return viewports[index];
}

void D3D11_Core::ClearBackBuffer()
{
  //static float clear_color[4] = { 0, 0, 0, 1.0f };
  static float clear_color[4] = { 0.0f, 0.125f, 0.3f, 1.0f };
  m_immediate_context->ClearRenderTargetView(m_render_target_view,clear_color);
}

void D3D11_Core::ClearDepthBuffer()
{
  m_immediate_context->ClearDepthStencilView(m_depth_stencil_view,
    D3D11_CLEAR_DEPTH,1.0f,0);
}

void D3D11_Core::DrawIndexedPrimitives(ID3D11DeviceContext* context,
  D3D11_PRIMITIVE_TOPOLOGY primitive,UINT start_index,UINT num_indices)
{
  // set the primitive topology
  context->IASetPrimitiveTopology(primitive);
  
  // draw
  context->DrawIndexed(num_indices,start_index,0);
}

void D3D11_Core::DrawInstancedPrimitives(ID3D11DeviceContext* context,
  D3D11_PRIMITIVE_TOPOLOGY primitive,UINT start_index,UINT num_indices,
  UINT start_instance,UINT num_instances)
{
  // set the primitive topology
  context->IASetPrimitiveTopology(primitive);
  
  // draw
  context->DrawIndexedInstanced(num_indices,num_instances,start_index,0,
    start_instance);
}

void D3D11_Core::Swap()
{
  m_swap_chain->Present(0,0);
}
